# This code creates CPI data for inflation adjustment

#.rs.restartR()
#(.packages())
# rm(list = ls())

library(RPostgreSQL)
library(dplyr)

pg <- dbConnect(PostgreSQL())

# Define PG data ----

# Project schema
rs <- dbGetQuery(pg, "SET search_path TO non_gaap")

# Outside of project schema
mcti <- tbl(pg, sql("SELECT * FROM crsp.mcti"))

# Create CPI data and write to PG ----

dbGetQuery(pg, "DROP TABLE IF EXISTS cpi")

# Adjust level data as of  2018-12-31  using CPI index
# CPI Adjustment factor X_Adj_2018-12-31 = X * cpifac
cpi_base <- 
    mcti %>% 
    select(caldt, cpiind) %>%
    filter(caldt == '2018-12-31') %>% 
    select(cpiind) %>% 
    collect() %>% 
    .[[1]]

cpi <- 
    mcti %>% 
    select(caldt, cpiind) %>%
    mutate(cpidt_base = sql("'2018-12-31'::date"),
           cpiind_base = cpi_base,
           cpifac = cpi_base/cpiind, 
           caldt_year = sql("date_part('year', caldt)"), 
           caldt_month = sql("date_part('month', caldt)")) %>%
    compute(name = "cpi", temporary=FALSE,
            indexes=list(c("caldt_year", "caldt_month")))

dbGetQuery(pg, "ALTER TABLE cpi OWNER TO non_gaap")
dbGetQuery(pg, "GRANT SELECT ON TABLE cpi TO non_gaap_access")
dbGetQuery(pg, "VACUUM ANALYZE cpi")

comment <- 'CREATED USING create_cpi.R  from GitHub azakolyukina/non_gaap'
db_comment <- paste0("COMMENT ON TABLE cpi IS '",
                     comment, " ON ", Sys.time() , "'; ") 
dbGetQuery(pg, db_comment)

dbDisconnect(pg)