function calc_moments(x,
                      delta, r_inv, coeff_damp, min_pts,
                      q_num, nu_y_num, m_y, nu_pi_num, m_pi,
                      exog_num, w_num, b_num,
                      policy_tol, price_tol, pshuffle,
                      howard_iter, max_iter, price_iter,
                      obs, t, tburn, sims, qseed, exog_uniform, shuffleshocks,
                      bunchtol, max_sim_bunch, wins,
                      mom_index, mom_labels, price_coef,
                      fixed_coef, psi_q_lb, q_space;
                      return_prices = false, counterfact = false, keepflags = true,
                      is_incl_psi = true, is_incl_b = true, is_incl_w = is_incl_w,
                      is_price_fixed = false, fixed_beta = missing,
                      is_comp_static = false, is_x_grid_x = true, x_grid = [], price_coef_grid = [])
    # This function calculates moments for a given set of parameter values

    # Set parameters to x vector
    alph, kappa_w, rho_y, sigma_y, sigma_pi, theta, kappa_b, mu_pi = x
    kappa_b = kappa_b*10.0 # WARNING: change kappa_b units

    # Set parameters used for grid spacing to x_grid vector if not empty
    if is_x_grid_x # if model parameters are same as those in the grid
        alph_grid, kappa_w_grid, rho_y_grid,
            sigma_y_grid, sigma_pi_grid, theta_grid, kappa_b_grid, mu_pi_grid = x
        price_coef_grid = price_coef
    else # if model parameters are different as those in the grid
        alph_grid, kappa_w_grid, rho_y_grid,
            sigma_y_grid, sigma_pi_grid, theta_grid, kappa_b_grid, mu_pi_grid = x_grid
    end
    kappa_b_grid = kappa_b_grid*10.0 # WARNING: change kappa_b units

    # Set grids
    qgrid, minq, maxq, distq, tran_mat, qmat,
        exogval, exogind, exogmat1, exogmat2,
        wgrid, wmax, bgrid, bmin, bmax, pol_num, pol_val, pol_old,
        w, b, v_old = init_grids(alph_grid, delta, rho_y_grid, sigma_y_grid, sigma_pi_grid, mu_pi_grid, r_inv,
                                 theta_grid, price_coef_grid, kappa_w_grid, kappa_b_grid,
                                 q_num, m_y, nu_y_num, m_pi, nu_pi_num,
                                 exog_num, b_num, w_num, is_incl_b, is_incl_psi, q_space)

    # Initialize flags
    sing_flag = 99
    vf_flag = 99
    flag_coef = 99
    price_flag = 99
    qmid_flag = 99
    rho_flag = 99

    bunch_flag = Array{Int}(undef, sims, 5) # bunch flags
    rho_flag_sim = Array{Int}(undef, sims, 1) # rho flag

    # Initialize bunch_flag and rho_flag at not being computed
    bunch_flag = repeat([99], sims, 5)  # bunch_flag
    rho_flag_sim = repeat([99], sims, 1) # rho_flag does not get computed

    # Post-burn-in simulated data
    sim_data = DataFrame(nsim = Int64[], firm = Int64[], time = Int64[],
    q = Float64[], w = Float64[], b = Float64[],
    nu_y = Float64[], nu_pi = Float64[],
    gaap = Float64[], non_gaap = Float64[],
    d = Float64[], cash = Float64[],
    vf = Float64[], price = Float64[])

    # Solve the model
    if !is_price_fixed
        # Use rational expectations

        # Set inital values for price coeffs.
        if is_incl_psi
            if is_incl_b
                # Price: {1, s_shape(pi + psi), s_shape(pi)}
                beta_old  = [mean(v_old), price_coef[2], price_coef[3]] # pricing coef.
            else
                # Price: {1, s_shape(pi + psi)}
                beta_old  = [mean(v_old), price_coef[2]] # pricing coef.
            end
        else
            # Price: {1, s_shape(pi)}
            beta_old  = [mean(v_old), price_coef[2]]
        end

        if is_incl_w
            # Append w coefficient to betas
            beta_old = [beta_old; price_coef[end]]
        end


        # First stage with price_coeff from data ----

        w_star, b_star, beta, price_flag, sing_flag, vf_flag, vf, v_star, vfvec, nongaapvec, gaapvec =
            rational_expect(tran_mat, exog_num, exogval, v_old,
                            pol_old, pol_num, pol_val,
                            w, b,
                            alph, delta, kappa_w, theta, kappa_b,
                            beta_old, coeff_damp,
                            r_inv, qmat, qgrid, minq, maxq, q_num, distq, exogmat1, exogmat2,
                            policy_tol, price_tol, howard_iter, max_iter, price_iter, fixed_coef,
                            is_incl_psi, is_incl_b, is_incl_w)

        # Check pricing slopes
        flag_coef = 0

        # WARNING: exit if anything breaks with RE
        if (price_flag != 0 || sing_flag != 0 || flag_coef != 0) && keepflags == true && counterfact == false
            # If various flags are triggered return undef moments

            rho_flag = maximum(rho_flag_sim)
            bunch_flag = [maximum(bunch_flag[:, 1]), maximum(bunch_flag[:, 2]),
            maximum(bunch_flag[:, 3]), maximum(bunch_flag[:, 4]),
            maximum(bunch_flag[:, 5])]
            moments = Array{Float64}(undef, length(mom_index))

            if return_prices
                return bunch_flag, price_flag, rho_flag, sing_flag, flag_coef, vf_flag, qmid_flag, moments, beta,
                sim_data, qgrid, wgrid, bgrid
            end

            return bunch_flag, price_flag, rho_flag, sing_flag, flag_coef, vf_flag, qmid_flag, moments, sim_data, qgrid, wgrid, bgrid
        end # else, continue onto simulating data and calculating moments

        # Second stage with price_coeff from the first stage ----

        # WARNING: only if price converged, i.e,. price_flag = 0 and using model parameters for grids
        if (price_flag == 0 && flag_coef == 0)

            # Re-set grids only if grid is not fixed
            if is_x_grid_x
                qgrid, minq, maxq, distq, tran_mat, qmat,
                    exogval, exogind, exogmat1, exogmat2,
                    wgrid, wmax, bgrid, bmin, bmax, pol_num, pol_val, pol_old,
                    w, b, v_old = init_grids(alph, delta, rho_y, sigma_y, sigma_pi, mu_pi, r_inv,
                                             theta, beta, kappa_w, kappa_b,
                                             q_num, m_y, nu_y_num, m_pi, nu_pi_num,
                                             exog_num, b_num, w_num, is_incl_b, is_incl_psi, q_space)
            end


            w_star, b_star, beta, price_flag, sing_flag, vf_flag, vf, v_star, vfvec, nongaapvec, gaapvec =
                rational_expect(tran_mat, exog_num, exogval, v_old,
                                pol_old, pol_num, pol_val,
                                w, b,
                                alph, delta, kappa_w, theta, kappa_b,
                                beta, coeff_damp,
                                r_inv, qmat, qgrid, minq, maxq, q_num, distq, exogmat1, exogmat2,
                                policy_tol, price_tol, howard_iter, max_iter, price_iter, fixed_coef,
                                is_incl_psi, is_incl_b, is_incl_w)

        end

        # Does steady-state q exists and, if exists, within grid bounds?
        # qmid_flag = 1 when steady-state q does NOT exists
        if is_x_grid_x # if model parameters are the same as grid parameters
            qmid, qmid_flag = calc_qmid(alph_grid, delta, kappa_w_grid,
                                        theta_grid, kappa_b_grid, mu_pi_grid,
                                        r_inv, beta)
        else  # if model parameters are different than grid parameters
            qmid, qmid_flag = calc_qmid(alph_grid, delta, kappa_w_grid,
                                        theta_grid, kappa_b_grid, mu_pi_grid,
                                        r_inv, price_coef_grid)
        end

        if !(minq < qmid && qmid < maxq)
            # Is steady-state q within qgrid boudns?
            qmid_flag = 1
        end

        # Check pricing slopes
        flag_coef = 0

    else
        # When price is fixed, just solve the firm's problem
        # WARNING: this will likely yield boundary solution for qgrid
        beta = fixed_beta

        w_star, b_star, val = model(tran_mat, exog_num, exogval, v_old,
                                    pol_old, pol_num, pol_val,
                                    w, b,
                                    alph, delta, kappa_w, theta, kappa_b, r_inv,
                                    fixed_beta,
                                    qmat, qgrid, minq, maxq, q_num, distq, exogmat1, exogmat2,
                                    policy_tol, howard_iter, max_iter, is_incl_psi, is_incl_b, is_incl_w)

        vf = fund_value(tran_mat, exog_num, exogval, w_star,
                        alph, delta, kappa_w, r_inv,
                        qgrid, minq, maxq, q_num, distq,
                        howard_iter, policy_tol)

        # Turn off flags that are only needed for RE
        sing_flag = 0
        flag_coef = 0
        price_flag = 0
        vf_flag = minimum(vf) < 0.0 ? 1 : 0
        qmid_flag = 0 # IGNORE qmid when price is fixed
    end

    # Return model results if using comparative statics based on model only
    if is_comp_static
        return qgrid, qmat, exogval, w_star, b_star, vf, beta, price_flag, sing_flag, vf_flag, qmid_flag
    end

    # println("Calculating simulated data")
    # WARNING: use mom_index to select moments to return
    nmom = length(mom_labels)

    # Turn off flags when computing counterfactuals so that moments can be calculated
    if counterfact; price_flag = 0; vf_flag = 0; end

    # Define output for the moment vector
    output = Array{Float64}(undef, nmom)

    if (price_flag != 0 || sing_flag != 0 || vf_flag != 0 || qmid_flag != 0) && keepflags == true
        # If various flags are triggered return undef moments

        rho_flag = maximum(rho_flag_sim)
        bunch_flag = [maximum(bunch_flag[:, 1]), maximum(bunch_flag[:, 2]),
        maximum(bunch_flag[:, 3]), maximum(bunch_flag[:, 4]),
        maximum(bunch_flag[:, 5])]
        moments = Array{Float64}(undef, length(mom_index))

        if return_prices
            return bunch_flag, price_flag, rho_flag, sing_flag, flag_coef, vf_flag, qmid_flag, moments, beta, sim_data, qgrid, wgrid, bgrid
        end

        return bunch_flag, price_flag, rho_flag, sing_flag, flag_coef, vf_flag, qmid_flag, moments, sim_data, qgrid, wgrid, bgrid
    end # else, continue onto simulating data and calculating moments

    # Initialize sim_moment vector
    sim_moment = Array{Float64}(undef, sims, nmom) # Simulated moments
    vf_mean = Array{Float64}(undef, sims, 1) # fundamental value
    b_mean = Array{Float64}(undef, sims, 1) # bias
    w_piw_mean = Array{Float64}(undef, sims, 1) # mean of w /q
    pi_piw_mean = Array{Float64}(undef, sims, 1) # mean of pi
    w_mean = Array{Float64}(undef, sims, 1) # level of investment
    q_mean = Array{Float64}(undef, sims, 1) # level of capital
    pi_mean = Array{Float64}(undef, sims, 1) # level of gaap
    vfq_mean = Array{Float64}(undef, sims, 1) # mean of vf / q
    bunchct = 0 # set bunch counter to 0

    # Loop through simulations
    for ss in 1:sims

        # Simulate data based on optimal actions
        qsim, nu_ysim, nu_pisim, wsim, bsim, vfsim, bunch_flag_sim =
            sim_model(obs, t, qseed[:, 1, ss], exog_uniform[:, :, ss],
                      maxq, minq, q_num, distq, qgrid,
                      wmax, bmax, bmin, delta, tburn,
                      exog_num, exogval, tran_mat, w_star, b_star, vf, bunchtol)

        # Turn off bias-relted flags for counterfactuals
        if (counterfact==1)
            bunch_flag_sim[4] = 0
            bunch_flag_sim[5] = 0
        end

        # Only compute moments if bunch_flag_sim is not triggered
        if maximum(bunch_flag_sim) < 1 || keepflags == false

            # Compute expected cash flows and "reshuffled" cash flows
            dsim = expect_div(nu_ysim, qsim, wsim, alph, kappa_w) # expected cash flow
            cfsim = dsim

            # Loop through observations to reshuffle cash flows
            for ii in 1:obs
                for jj in 2:t-1

                    # Amount of cf to allocate to previous or next period (if next, take negative)
                    shuffleval = pshuffle*2.0*(0.5 - shuffleshocks[ii, jj, ss])*dsim[ii, jj]

                    # If shock is <0.5 then shuffle backward
                    if shuffleshocks[ii, jj, ss] < 0.5
                        cfsim[ii, jj - 1] = dsim[ii, jj - 1] + shuffleval

                        # If shock is >=0.5 then shuffle forward
                    else
                        shuffleval *= -1
                        cfsim[ii, jj + 1] = dsim[ii, jj + 1] + shuffleval
                    end

                    # Subtract that amount from current period
                    cfsim[ii, jj] = dsim[ii, jj] - shuffleval
                end
            end

            # Compute other variables for moments
            gaapsim    = dsim .+ qsim.*nu_pisim # GAAP earnings
            gaapsim2    = gaapsim.^2.0 # GAAP earnings^2
            ngadjustsim = non_gaap(bsim, qsim, nu_pisim) # non-GAAP adj.
            ngadjustsim2 = ngadjustsim.^2.0 # non-GAAP adj.^2
            nongaapsim = ngadjustsim .+ gaapsim # non-GAAP earnings
            gaapngadjustsim = gaapsim.*ngadjustsim # interaction between GAAP and non-GAAP


            # Compute price
            gaapsim_vec = vec(gaapsim)
            nongaapsim_vec = vec(nongaapsim)
            if is_incl_psi
                if is_incl_b
                    pricesim_vec = beta[1] .+ beta[2].*s_shape(nongaapsim_vec) .+ beta[3].*s_shape(gaapsim_vec)
                else
                    pricesim_vec = beta[1] .+ beta[2].*s_shape(nongaapsim_vec)
                end
            else
                pricesim_vec = beta[1] .+ beta[2].*s_shape(gaapsim_vec)
            end

            if is_incl_w
                wsim_vec = vec(wsim)
                pricesim_vec = pricesim_vec .+ beta[end].*wsim_vec
            end

            pricesim = reshape(pricesim_vec, obs, t)
            deltaprice = pricesim[:, tburn:t-1] .- pricesim[:, tburn-1:t-2] # change in price

            # Compute variables scaled by q
            wsimq       = wsim./qsim
            gaapsimq    = gaapsim./qsim    # gaap / q
            ngadjustsimq = ngadjustsim./qsim # non-gaap adjustment / q
            nongaapsimq = nongaapsim./qsim # non-gaap earnings / q
            cfsimq      = cfsim./qsim      # cash flow / q
            vfsimq       = vfsim./qsim      # vf / q

            # Compute growth rates from t-1 and convert to vectors
            wgrowth = vec(2.0*(wsim[:, tburn+1:t-1] - wsim[:, tburn:t-2])./
                (abs.(wsim[:, tburn+1:t-1]) + abs.(wsim[:, tburn:t-2]))) # Change in investment
            cfgrowth = vec(2.0*(cfsim[:, tburn+1:t-1] - cfsim[:, tburn:t-2])./
                (abs.(cfsim[:, tburn+1:t-1]) .+ abs.(cfsim[:, tburn:t-2]))) # Change in cash flows
            gaapgrowth = vec(2.0*(gaapsim[:, tburn+1:t-1] - gaapsim[:, tburn:t-2])./
                (abs.(gaapsim[:, tburn+1:t-1]) .+ abs.(gaapsim[:, tburn:t-2]))) # Change in GAAP earnings
            gaapgrowth2 = vec(2.0*(gaapsim2[:, tburn+1:t-1] - gaapsim2[:, tburn:t-2])./
                (abs.(gaapsim2[:, tburn+1:t-1]) .+ abs.(gaapsim2[:, tburn:t-2]))) # Change in GAAP earnings squared
            ngadjustgrowth = vec(2.0*(ngadjustsim[:, tburn+1:t-1] - ngadjustsim[:, tburn:t-2])./
                (abs.(ngadjustsim[:, tburn+1:t-1]) .+ abs.(ngadjustsim[:, tburn:t-2]))) # Change in non-gaap adjustment
            ngadjustgrowth2 = vec(2.0*(ngadjustsim2[:, tburn+1:t-1] - ngadjustsim2[:, tburn:t-2])./
                (abs.(ngadjustsim2[:, tburn+1:t-1]) .+ abs.(ngadjustsim2[:, tburn:t-2]))) # Change in non-gaap adjustment squared
            gaapnongaapadjgrowth = vec(2.0*(gaapngadjustsim[:, tburn+1:t-1] - gaapngadjustsim[:, tburn:t-2])./
                (abs.(gaapngadjustsim[:, tburn+1:t-1]) .+ abs.(gaapngadjustsim[:, tburn:t-2]))) # Change in GAAP x non-GAAP adjustment
            nongaapgrowth = vec(2.0*(nongaapsim[:, tburn+1:t-1] - nongaapsim[:, tburn:t-2])./
                (abs.(nongaapsim[:, tburn+1:t-1]) .+ abs.(nongaapsim[:, tburn:t-2]))) # Change in non-GAAP
            deltapricegrowth = vec(2.0*(deltaprice[:, 2:end] - deltaprice[:, 1:end-1])./
                (abs.(deltaprice[:, 2:end]) .+ abs.(deltaprice[:, 1:end-1]))) # Change in change in price

            # Set instances when growth rates are NaN to 0
            for ii in 1:length(wgrowth)
                if isnan(wgrowth[ii]) # w
                    wgrowth[ii] = 0.0
                end
                if isnan(cfgrowth[ii]) # cf
                    cfgrowth[ii] = 0.0
                end
                if isnan(gaapgrowth[ii]) # gaap
                    gaapgrowth[ii] = 0.0
                end
                if isnan(gaapgrowth2[ii]) # gaap2
                    gaapgrowth2[ii] = 0.0
                end
                if isnan(ngadjustgrowth[ii]) # nonGAAP adjustment
                    ngadjustgrowth[ii] = 0.0
                end
                if isnan(ngadjustgrowth2[ii]) # nonGAAP adjustment2
                    ngadjustgrowth2[ii] = 0.0
                end
                if isnan(gaapnongaapadjgrowth[ii]) # gaapnonGAAP
                    gaapnongaapadjgrowth[ii] = 0.0
                end
                if isnan(nongaapgrowth[ii]) # nonGAAP
                    nongaapgrowth[ii] = 0.0
                end
                if isnan(deltapricegrowth[ii]) # firm value
                    deltapricegrowth[ii] = 0.0
                end
            end

            # Keep post-burn-in data to calculate mean moments and conver to vectors
            wsimq_post       = vec(wsimq[:,tburn+1:t-1]) # investment / q
            gaapsimq_post    = vec(gaapsimq[:,tburn+1:t-1]) # GAAP earnings / q
            ngadjustsimq_post= vec(ngadjustsimq[:,tburn+1:t-1]) # non-GAAP adj. / q
            nongaapsimq_post = vec(nongaapsimq[:, tburn+1:t-1]) # non-GAAP earnings / q
            cfsimq_post      = vec(cfsimq[:, tburn+1:t-1]) # shuffled cash flow  / q
            vfsimq_post      = vec(vfsimq[:, tburn+1:t-1]) # price / q
            vfsim_post       = vec(vfsim[:, tburn+1:t-1]) # price
            gaapsim_post     = vec(gaapsim[:, tburn+1:t-1]) # GAAP earings
            nongaapsim_post  = vec(nongaapsim[:, tburn+1:t-1]) # non-GAAP earnings
            ngadjustsim_post = vec(ngadjustsim[:,tburn+1:t-1]) # non-GAAP adj.

            # Create indicator for whether the non-GAAP adj. is positive AND large, i.e., > psi_q_lb
            psipos = ngadjustsimq_post .> psi_q_lb
            if (sum(psipos) == 0)
                # WARNING: prevent failure for conditioning when none are large enough
                psipos[1] = 1
            end

            # winsor_mzize data
            if wins > 0.0
                # Amounts scaled by q
                wsimq_post = collect(winsor_mz(wsimq_post, prop = wins)) # investment / q
                gaapsimq_post = collect(winsor_mz(gaapsimq_post, prop = wins)) # gaap / q
                ngadjustsimq_post = collect(winsor_mz(ngadjustsimq_post, prop = wins)) # non-gaap adjustment / q
                nongaapsimq_post = collect(winsor_mz(nongaapsimq_post, prop = wins)) # non-gaap / q
                cfsimq_post = collect(winsor_mz(cfsimq_post, prop = wins)) # cash flow / q
                vfsimq_post = collect(winsor_mz(vfsimq_post, prop = wins)) # vf / q

                # Growth rates
                wgrowth = collect(winsor_mz(wgrowth, prop = wins)) # Change in investment
                cfgrowth = collect(winsor_mz(cfgrowth, prop = wins)) # Change in cash flows
                gaapgrowth = collect(winsor_mz(gaapgrowth, prop = wins)) # Change in GAAP earnings
                gaapgrowth2 = collect(winsor_mz(gaapgrowth2, prop = wins)) # Change in GAAP earnings squared
                ngadjustgrowth = collect(winsor_mz(ngadjustgrowth, prop = wins)) # Change in non-GAAP adjustment
                ngadjustgrowth2 = collect(winsor_mz(ngadjustgrowth2, prop = wins)) # Change in non-GAAP adjustment, squared
                gaapnongaapadjgrowth = collect(winsor_mz(gaapnongaapadjgrowth, prop = wins)) # Change in GAAP x non-GAAP adjustment
                nongaapgrowth = collect(winsor_mz(nongaapgrowth, prop = wins)) # Change in non-GAAP
                deltapricegrowth = collect(winsor_mz(deltapricegrowth, prop = wins)) # Change in price
            end
            # Compute mean moments
            mom_wq   = mean(wsimq_post) # investment / q
            mom_piq  = mean(gaapsimq_post) # gaap / q
            mom_psiq = mean(ngadjustsimq_post) # non-gaap adjustment / q
            mom_dq   = mean(cfsimq_post) # cash flow / q
            mom_psipos = mean(psipos) # psi positive
            mom_psiq_psipos = mean_cond(ngadjustsimq_post, psipos) # non-gaap adjustment / q positive psi
            mom_piq_psipos = mean_cond(gaapsimq_post, psipos) # gaap / q positive psi

            # Compute AR1 moments (i.e., the presistence coefficient)
            mom_rho_piq  = ar1(gaapsimq_post, obs) # gaap
            mom_rho_psiq = ar1(ngadjustsimq_post, obs) # non-gaap
            mom_rho_dq   = ar1(cfsimq_post, obs) # cash flow

            # Check if any AR1 > max_permitted_AR value
            ar_max = 0.975
            if (mom_rho_piq >= ar_max)
                rho_flag_sim[ss, 1] = 1
            else
                rho_flag_sim[ss, 1] = 0
            end

            # Compute regression coefficient for earnings on lagged non-GAAP earnings
            non_gaap_coef = reg_lag(gaapsimq_post, nongaapsimq_post, obs) # predictive power of non-GAAP

            # Compute covariance matrix of growth rates
            cov_growth = cov([wgrowth gaapgrowth cfgrowth ngadjustgrowth deltapricegrowth gaapgrowth2 ngadjustgrowth2 gaapnongaapadjgrowth nongaapgrowth])

            # Compute conditional covariances
            mom_w_w_psipos = cov_cond(wgrowth, wgrowth, psipos) # w growth, positive psi
            mom_w_pi_psipos = cov_cond(wgrowth, gaapgrowth, psipos) # w and pi, positive psi
            mom_w_d_psipos = cov_cond(wgrowth, cfgrowth, psipos) # w and d, positive psi
            mom_w_psi_psipos = cov_cond(wgrowth, ngadjustgrowth, psipos) # w and psi, positive psi
            mom_pi_pi_psipos = cov_cond(gaapgrowth, gaapgrowth, psipos) # pi, positive psi
            mom_pi_d_psipos = cov_cond(gaapgrowth, cfgrowth, psipos) # pi and d, positive psi
            mom_pi_psi_psipos = cov_cond(gaapgrowth, ngadjustgrowth, psipos) # pi and psi, positive psi
            mom_d_d_psipos = cov_cond(cfgrowth, cfgrowth, psipos) # d, positive psi
            mom_d_psi_psipos = cov_cond(cfgrowth, ngadjustgrowth, psipos) # d and psi, positive psi
            mom_psi_psi_psipos = cov_cond(ngadjustgrowth, ngadjustgrowth, psipos) # psi and psi, positive psi
            mom_dvf_pi_psipos = cov_cond(deltapricegrowth, gaapgrowth, psipos) # prc and pi, positive psi
            mom_dvf_ping_psipos = cov_cond(deltapricegrowth, nongaapgrowth, psipos) # prc and psi + pi, positive psi

            # Conidtional skew
            # WARNING: may NOT compute because of x**(3/2)
            mom_skew_psiq_psipos = try skew_cond(ngadjustsimq_post, psipos) catch err; -99.0 end

            # Compute beta coefficients from simulated data for price regression
            # vf/q = b0 + b1*s_shape((pi + psi)/q) + b2*s_shape(pi/q)
            one_vec = ones(length(vfsimq_post)) # vector of ones
            nongaapsimq_s_shape = s_shape(nongaapsimq_post) # S-shape of non-GAAP earnings
            gaapsimq_s_shape = s_shape(gaapsimq_post) # S-shape of GAAP earnings
            xsim_reg = [one_vec nongaapsimq_s_shape gaapsimq_s_shape] # combine gaap and non-gaap into a matrix for regression
            mom_beta = try inv(xsim_reg'*xsim_reg)*xsim_reg'*vfsimq_post catch err; [-99.0; -99.0] end

            # Create indicator for whether the non-GAAP adj. is zero
            mom_psizero = mean(ngadjustsim_post .== 0.0)

            # Combine to create moment vector
            sim_moment[ss, :] = [mom_wq; # E(w/q)
                                 mom_piq; # E(pi/p)
                                 mom_rho_piq; # rho(pi/q)
                                 mom_dq; # E(d/q)
                                 mom_rho_dq; # rho(d/q)
                                 mom_psiq; # E(psi/q)
                                 mom_rho_psiq; # rho(psi/q)
                                 cov_growth[1, 1];  # Cov(Dw, Dw)
                                 cov_growth[1, 2];  # Cov(Dw, Dpi)
                                 cov_growth[1, 3];  # Cov(Dw, Dd)
                                 cov_growth[1, 4];  # Cov(Dw, Dpsi)
                                 cov_growth[2, 2];  # Cov(Dpi, Dpi)
                                 cov_growth[2, 3];  # Cov(Dpi, Dd)
                                 cov_growth[2, 4];  # Cov(Dpi, Dpsi)
                                 cov_growth[3, 3];  # Cov(Dd, Dd)
                                 cov_growth[3, 4];  # Cov(Dd, Dpsi)
                                 cov_growth[4, 4];  # Cov(Dpsi, Dpsi)
                                 non_gaap_coef; # Regression coefficient of non-GAAP on future GAAP
                                 cov_growth[2, 5];  # Cov(Dpi, DDprice)
                                 cov_growth[4, 5];  # Cov(Dpsi, DDprice)
                                 cov_growth[6, 5];  # Cov(Dpi2, DDprice)
                                 cov_growth[7, 5];  # Cov(Dpsi2, DDprice)
                                 cov_growth[8, 5]; # Cov(Dpipsi, DDprice)
                                 cov_growth[9, 5]; # Cov(D(pi+psi), DDprice)
                                 mom_psipos; # "E(psipos)"
                                 mom_psiq_psipos; # E(psi/q | psipos)
                                 mom_w_w_psipos; # Cov(Dw, Dw | psipos)
                                 mom_w_pi_psipos; # Cov(Dw, Dpi | psipos)
                                 mom_w_d_psipos; # Cov(Dw, Dd | psipos)
                                 mom_w_psi_psipos; # Cov(Dw, Dpsi | psipos)
                                 mom_pi_pi_psipos; # Cov(Dpi, Dpi | psipos)
                                 mom_pi_d_psipos; # Cov(Dpi, Dd | psipos)
                                 mom_pi_psi_psipos; # Cov(Dpi, Dpsi | psipos)
                                 mom_d_d_psipos; # Cov(Dd, Dd | psipos)
                                 mom_d_psi_psipos; # Cov(Dd, Dpsi | psipos)
                                 mom_psi_psi_psipos; # Cov(Dpsi, Dpsi | psipos)
                                 mom_dvf_pi_psipos; # Cov(DDvf, Dpi | psipos)
                                 mom_dvf_ping_psipos; # Cov(DDvf, D(psi + pi) | psipos)
                                 mom_skew_psiq_psipos; # Skew(psi/q | psipos)
                                 mom_beta[2]; # beta_pipsi
                                 mom_beta[3]; # beta_pi
                                 mom_psizero; # E(psi==0)
                                 mom_piq_psipos] # E(pi/q | psipos)

            # Compute post-burn-in simulated data ----

            # Keep post-burn-in data to calculate mean moments and convert to vectors
            qsim_post = vec(qsim[:,tburn+1:t-1]) # Capital
            wsim_post = vec(wsim[:,tburn+1:t-1]) # Investment
            bsim_post = vec(bsim[:,tburn+1:t-1]) # Bias
            gaapsim_post = vec(gaapsim[:,tburn+1:t-1]) # GAAP earnings
            nongaapsim_post = vec(nongaapsim[:, tburn+1:t-1]) # non-GAAP earnings
            cfsim_post = vec(cfsim[:, tburn+1:t-1]) # Re-shuffled cash flow
            dsim_post = vec(dsim[:, tburn+1:t-1]) # Dividends
            vfsim_post = vec(vfsim[:, tburn+1:t-1]) # Firm value
            pricesim_post = vec(pricesim[:, tburn+1:t-1]) # Firm value or price
            nu_ysim_post = vec(nu_ysim[:, tburn+1:t-1]) # nu_y
            nu_pisim_post = vec(nu_pisim[:, tburn+1:t-1]) # nu_pi

            # Create firm and time indicators
            nsim = ss*ones(length(qsim_post))
            firm = repeat(1:obs, t - (tburn + 1))
            time = repeat((tburn + 1):(t - 1), inner = obs)

            # Save post-burn-in simulated data
            temp = DataFrame(nsim = nsim, firm = firm, time = time,
                             q = qsim_post, w = wsim_post, b = bsim_post,
                             nu_y = nu_ysim_post, nu_pi = nu_pisim_post,
                             gaap = gaapsim_post, non_gaap = nongaapsim_post,
                             d = dsim_post, cash = cfsim_post,
                             vf = vfsim_post, price = pricesim_post)

            sim_data = [sim_data; temp]

            # If bunch_flag_sim is triggered, increment bunchct
        else
            bunchct = bunchct + 1 # add to bunch count of simulations
            bunch_flag[ss, :] = bunch_flag_sim
        end
    end

    # Determine whether any simulations triggered rho_flag
    rho_flag = maximum(rho_flag_sim)

    # Take average across simulations if fewer bunched than count tolerance
    if (bunchct <= max_sim_bunch)

        # Cycle through simulations, only keeping those without bunch_flags
        output = sum(sim_moment, dims = 1)' ./ (sims - bunchct)

        bunch_flag = repeat([0], sims, 5) # refresh bunch_flag
        bunch_flag = [maximum(bunch_flag[:, 1]), maximum(bunch_flag[:, 2]),
        maximum(bunch_flag[:, 3]), maximum(bunch_flag[:, 4]),
        maximum(bunch_flag[:, 5])]

        # Calculate moments if we ignore flags
    elseif keepflags == false
        output = sum(sim_moment, dims = 1)' ./ sims
        bunch_flag = [maximum(bunch_flag[:, 1]), maximum(bunch_flag[:, 2]),
        maximum(bunch_flag[:, 3]), maximum(bunch_flag[:, 4]),
        maximum(bunch_flag[:, 5])]
    else
        bunch_flag = [maximum(bunch_flag[:, 1]), maximum(bunch_flag[:, 2]),
        maximum(bunch_flag[:, 3]), maximum(bunch_flag[:, 4]),
        maximum(bunch_flag[:, 5])]
    end

    # Keep selected moments
    moments = output[mom_index]

    # For printing in the objective function
    if return_prices
        return bunch_flag, price_flag, rho_flag, sing_flag, flag_coef, vf_flag, qmid_flag, moments, beta, sim_data, qgrid, wgrid, bgrid, w_star, b_star, exogval
    end

    # Return moments, flags, and simulated data
    # bunch_flag: integer to indicate whether flag was tripped.
    # price_flag: integer to indicate whether price iterations were exceeded
    return bunch_flag, price_flag, rho_flag, sing_flag, flag_coef, vf_flag, qmid_flag, moments, sim_data, qgrid, wgrid, bgrid, w_star, b_star, exogval, vf, vfvec, nongaapvec, gaapvec
end
