function model(tran_mat, # {nu_y_num x nu_pi_num} transition matrix for socks
               exog_num, # number of points on grid along exogenous dimensions
               exogval, # {(nu_y_num*nu_pi_num) x 2} matrix of exogenous values
               v, # value function (manager)
               pol_old, # policies old
               pol_num, # number of policies
               pol_val, # value of policies
               w, # investment
               b, # bias
               alph, delta, kappa_w, theta, kappa_b, r_inv, # params
               beta, # pricing coeff
               qmat, # capital q matrix
               qgrid, # capital q grid
               minq, maxq, q_num, distq, # params of q grid
               exogmat1, exogmat2, # matrices of exog. params
               policy_tol, howard_iter, max_iter, is_incl_psi, is_incl_b, is_incl_w)
    # This function performs two-step policy iteration for the manager's problem

    # Initialize iteration counter and distance
    dist = Inf
    iter = 0

    # println("................................................")
    # println("Starting policy iteration...")

    # Loop through policy and value iterations
    while (iter < max_iter) && (dist > 0)

        # Value function holding controls fixed
        v = howard_accel(tran_mat, exog_num, exogval, v, w, b,
                         alph, delta, kappa_w, theta, kappa_b, r_inv,
                         beta,
                         qmat, qgrid, minq, maxq, q_num, distq,
                         exogmat1, exogmat2,
                         policy_tol, howard_iter, is_incl_psi, is_incl_b, is_incl_w)

        # Update policy functions
        pol_new = @distributed (hcat) for ii = 1:q_num
            optimal_policy(ii, tran_mat, exog_num, exogval, v, pol_num, pol_val,
                            alph, delta, kappa_w, theta, kappa_b, r_inv,
                            beta,
                            qgrid, minq, maxq, q_num, distq, is_incl_psi, is_incl_b, is_incl_w)
        end

        # Extract policies
        w = pol_val[pol_new, 1];
        b = pol_val[pol_new, 2];

        # Calculate distances between successive policy funtions and iteration count
        iter += 1;
        dist = maximum(abs.(pol_new - pol_old)); # distance between policies

        # Update value and policy function
        v_old = v; # value function
        pol_old = pol_new; # matrix of policies
    end

    # Return policies and value
    return w, b, v
end
